@@ -111,6 +111,12 @@ def get_cli_parser() -> argparse.ArgumentParser:
                         default=False,
                         action='store_true',
                         help="parseable output including severity of rule")
+    parser.add_argument('--progressive', dest='progressive',
+                        default=False,
+                        action='store_true',
+                        help="Return success if it detects a reduction in number"
+                        " of violations compared with previous git commit. This "
+                        "feature works only in git repositories.")
     parser.add_argument('-r', action=AbspathArgAction, dest='rulesdir',
                         default=[], type=Path,
                         help="Specify custom rule directories. Add -R "
