@@ -1,5 +1,6 @@
 """Utility functions related to file operations."""
 import os
+from contextlib import contextmanager
 
 
 def normpath(path) -> str:
@@ -11,3 +12,14 @@ def normpath(path) -> str:
     """
     # convertion to string in order to allow receiving non string objects
     return os.path.relpath(str(path))
+
+
+@contextmanager
+def cwd(path):
+    """Context manager for temporary changing current working directory."""
+    old_pwd = os.getcwd()
+    os.chdir(path)
+    try:
+        yield
+    finally:
+        os.chdir(old_pwd)
