@@ -40,7 +40,7 @@ from ansible.parsing.splitter import split_args
 from ansible.parsing.yaml.constructor import AnsibleConstructor
 from ansible.parsing.yaml.loader import AnsibleLoader
 from ansible.parsing.yaml.objects import AnsibleSequence
-from ansible.plugins.loader import module_loader
+from ansible.plugins.loader import add_all_plugin_dirs
 from ansible.template import Templar
 from yaml.composer import Composer
 from yaml.representer import RepresenterError
@@ -147,9 +147,27 @@ def _rebind_match_filename(filename: str, func) -> Callable:
     return func_wrapper
 
 
+def _set_collections_basedir(basedir: str):
+    # Sets the playbook directory as playbook_paths for the collection loader
+    try:
+        # Ansible 2.10+
+        # noqa: # pylint:disable=cyclic-import,import-outside-toplevel
+        from ansible.utils.collection_loader import AnsibleCollectionConfig
+
+        AnsibleCollectionConfig.playbook_paths = basedir
+    except ImportError:
+        # Ansible 2.8 or 2.9
+        # noqa: # pylint:disable=cyclic-import,import-outside-toplevel
+        from ansible.utils.collection_loader import set_collection_playbook_paths
+
+        set_collection_playbook_paths(basedir)
+
+
 def find_children(playbook: Tuple[str, str], playbook_dir: str) -> List:
     if not os.path.exists(playbook[0]):
         return []
+    _set_collections_basedir(playbook_dir or '.')
+    add_all_plugin_dirs(playbook_dir or '.')
     if playbook[1] == 'role':
         playbook_ds = {'roles': [{'role': playbook[0]}]}
     else:
@@ -205,8 +223,7 @@ def play_children(basedir, item, parent_type, playbook_dir):
         'import_tasks': _include_children,
     }
     (k, v) = item
-    play_library = os.path.join(os.path.abspath(basedir), 'library')
-    _load_library_if_exists(play_library)
+    add_all_plugin_dirs(os.path.abspath(basedir))
 
     if k in delegate_map:
         if v:
@@ -235,6 +252,11 @@ def _include_children(basedir, k, v, parent_type):
 def _taskshandlers_children(basedir, k, v, parent_type: FileType) -> List:
     results = []
     for th in v:
+
+        # ignore empty tasks, `-`
+        if not th:
+            continue
+
         with contextlib.suppress(LookupError):
             children = _get_task_handler_children_for_tasks_or_playbooks(
                 th, basedir, k, parent_type,
@@ -267,9 +289,16 @@ def _get_task_handler_children_for_tasks_or_playbooks(
 ) -> dict:
     """Try to get children of taskhandler for include/import tasks/playbooks."""
     child_type = k if parent_type == 'playbook' else parent_type
+
     task_include_keys = 'include', 'include_tasks', 'import_playbook', 'import_tasks'
     for task_handler_key in task_include_keys:
+
         with contextlib.suppress(KeyError):
+
+            # ignore empty tasks
+            if not task_handler:
+                continue
+
             return {
                 'path': path_dwim(basedir, task_handler[task_handler_key]),
                 'type': child_type,
@@ -310,11 +339,6 @@ def _roles_children(basedir: str, k, v, parent_type: FileType, main='main') -> l
     return results
 
 
-def _load_library_if_exists(path: str) -> None:
-    if os.path.exists(path):
-        module_loader.add_directory(path)
-
-
 def _rolepath(basedir: str, role: str) -> Optional[str]:
     role_path = None
 
@@ -345,7 +369,7 @@ def _rolepath(basedir: str, role: str) -> Optional[str]:
             break
 
     if role_path:
-        _load_library_if_exists(os.path.join(role_path, 'library'))
+        add_all_plugin_dirs(role_path)
 
     return role_path
 
@@ -546,6 +570,9 @@ def extract_from_list(blocks, candidates):
 def add_action_type(actions, action_type):
     results = list()
     for action in actions:
+        # ignore empty task
+        if not action:
+            continue
         action['__ansible_action_type__'] = BLOCK_NAME_TO_ACTION_TYPE_MAP[action_type]
         results.append(action)
     return results
@@ -752,6 +779,7 @@ def get_playbooks_and_roles(options=None) -> List[str]:  # noqa: C901
         elif 'roles' in p.parts or '.' in role_dirs:
             if 'tasks' in p.parts and p.parts[-1] in ['main.yaml', 'main.yml']:
                 role_dirs.append(str(p.parents[1]))
+                continue
             elif role_internals.intersection(p.parts):
                 continue
             elif 'tests' in p.parts:
